; QueueUserAPCEx Installer
; Copyright (c) 2011 Charles Wilson
;
; Permission is hereby granted, free of charge, to any person obtaining a
; copy of this software and associated documentation files (the "Software"),
; to deal in the Software without restriction, including without limitation
; the rights to use, copy, modify, merge, publish, distribute, sublicense,
; and/or sell copies of the Software, and to permit persons to whom the
; Software is furnished to do so, subject to the following conditions:
;
; The above copyright notice and this permission notice shall be included
; in all copies or substantial portions of the Software.
;
; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
; OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
; FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
; THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
; OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
; ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
; OTHER DEALINGS IN THE SOFTWARE.
;
; Note that ONLY this InnoSetup installer is licensed as above. The
; driver that it installs carry its own licensing terms (LGPLv2+).
; Furthermore, the included "services.iss" file carries its own Mozilla
; Public License.
;
; WARNING!!! This script must be compiled with the ANSI version (not
; unicode) of InnoSetup 5.3 or above.

#define MyAppName "QueueUserAPCEx"
#define MyAppVersion "20110505"
#define MyAppPublisher "MinGW"           ; "pthreads-win32"
#define MyAppURL "http://www.mingw.org/" ; "http://sourceware.org/pthreads-win32/"
#define MyDriverName        "AlertDrv"
#define MyDriverDisplayName "AlertDrv (Pthreads-Win32/QueueUserAPCEx)"
#define MyDriverFileName    "alertdrv.sys"

[Setup]
AppId={{51999A15-FA16-475F-90BE-7A2834F9E66A}}
AppName=QueueUserAPCEx
AppVersion={#MyAppVersion}
AppVerName={#MyAppName} {#MyAppVersion}
AppPublisher={#MyAppPublisher}
AppPublisherURL={#MyAppURL}
AppSupportURL={#MyAppURL}
AppUpdatesURL={#MyAppURL}
LicenseFile =QueueUserAPCEx/COPYING.LIB
DefaultDirName={pf}\PthreadsWin32\QueueUserAPCEx
DefaultGroupName=PthreadsWin32\QueueUserAPCEx
VersionInfoDescription={#MyAppName} v{#MyAppVersion} Setup
VersionInfoCompany={#MyAppPublisher}
VersionInfoTextVersion={#MyAppVersion}
;VersionInfoVersion={#MyAppVersion}
AllowNoIcons=yes
Compression =lzma
SolidCompression =yes
OutputDir =.
OutputBaseFilename ={#MyAppName}-install
; requires WinXP, or higher
MinVersion =0, 5.1
PrivilegesRequired=admin

; "ArchitecturesInstallIn64BitMode=x64 ia64" requests that the install
; be done in "64-bit mode" on x64 & Itanium, meaning it should use the
; native 64-bit Program Files directory and the 64-bit view of the
; registry. On all other architectures it will install in "32-bit mode".
ArchitecturesInstallIn64BitMode=x64 ia64

[Files]
Source: QueueUserAPCEx/driver/ChangeLog; DestDir: {app}; Flags: ignoreversion
Source: QueueUserAPCEx/Readme.txt; DestDir: {app}; Flags: ignoreversion
Source: README.MinGW; DestDir: {app}; Flags: ignoreversion
Source: QueueUserAPCEx/driver/alertdrv.reg; DestDir: {app}; Flags: ignoreversion
Source: QueueUserAPCEx/COPYING.LIB; DestDir: {app}; DestName: COPYING_LIB.txt; Flags: ignoreversion

; x86 (Windows XP and greater)
Source: QueueUserAPCEx/driver/objfre_wxp_x86/i386/alertdrv.sys; DestDir: {sys}/drivers; Flags: uninsneveruninstall replacesameversion restartreplace promptifolder; Check: IsX86
Source: QueueUserAPCEx/testapp/Release/Win32/testapp.exe; DestDir: {app}; Check: IsX86

; AMD 64bit
Source: QueueUserAPCEx/driver/objfre_wlh_amd64/amd64/alertdrv.sys; DestDir: {sys}/drivers; Flags: uninsneveruninstall replacesameversion restartreplace promptifolder; Check: IsX64
Source: QueueUserAPCEx/testapp/Release/x64/testapp.exe; DestDir: {app}; Check: IsX64

; Itanium 64bit
Source: QueueUserAPCEx/driver/objfre_wlh_amd64/amd64/alertdrv.sys; DestDir: {sys}/drivers; Flags: uninsneveruninstall replacesameversion restartreplace promptifolder; Check: IsI64
Source: QueueUserAPCEx/testapp/Release/Itanium/testapp.exe; DestDir: {app}; Check: IsI64

[Icons]
Name: {group}\README.MinGW; Filename: {app}\README.MinGW
Name: {group}\Readme.txt; Filename: {app}\Readme.txt
Name: {group}\LGPL License; Filename: {app}\COPYING_LIB.txt
Name: {group}\Uninstall QueueUserAPCEx; Filename: {uninstallexe}
Name: {group}\Run TestApp; Filename: {app}\testapp.exe

[Code]
#include "services.iss"
#include "devices.iss"

function IsX64: Boolean;
begin
  Result := Is64BitInstallMode and (ProcessorArchitecture = paX64);
end;

function IsI64: Boolean;
begin
  Result := Is64BitInstallMode and (ProcessorArchitecture = paIA64);
end;

function IsX86: Boolean;
begin
  Result := not IsX64 and not IsI64;
end;

function Is64: Boolean;
begin
  Result := IsX64 or IsI64;
end;

function UpdateReadyMemo(Space, NewLine, MemoUserInfoInfo, MemoDirInfo, MemoTypeInfo,
  MemoComponentsInfo, MemoGroupInfo, MemoTasksInfo: String): String;
var
  S: String;
begin
  { Add info about VC2010 runtimes to 'Ready Memo' }
  S := '';
  S := S + 'Note that the test application may not work correctly' + NewLine;
  S := S + 'if you do not have the appropriate Microsoft C++ runtime' + NewLine;
  S := S + 'installed.  These can be obtained from:' + NewLine + NewLine;
  S := S + 'Microsoft Visual C++ 2010 Redistributable Package (x86)' + NewLine;
  S := S + 'http://www.microsoft.com/downloads/en/details.aspx?FamilyID=a7b7a05e-6de6-4d3a-a423-37bf0912db84' + NewLine + NewLine;
  S := S + 'Microsoft Visual C++ 2010 Redistributable Package (x64)' + NewLine;
  S := S + 'http://www.microsoft.com/downloads/en/details.aspx?FamilyID=BD512D9E-43C8-4655-81BF-9350143D5867' + NewLine + NewLine;
  S := S + 'Microsoft Visual C++ 2010 Redistributable Package (ia64)' + NewLine;
  S := S + 'http://www.microsoft.com/downloads/en/details.aspx?FamilyID=1a2df53a-d8f4-4bfe-be35-152c5d3d0f82' + NewLine;
  Result := S;
end;

procedure UninstallDevice();
var
  aName: String;
  dName: String;
  access: Longword;

begin
  aName := '{#emit MyDriverName}';
  dName := '{#emit MyDriverDisplayName}';
  access := (SERVICE_START + DELETE_ACCESS_RIGHT + SERVICE_STOP + SERVICE_INTERROGATE);

  if DeviceExists(aName, access) = False then
    begin
      Exit;
    end
  else
    begin
      SimpleStopDevice(aName, access, True, True);
      SimpleDeleteDevice(aName, access);
    end;
end;

procedure InstallDevice();
var
  aName: String;
  dName: String;
  fName: String;
  sPath: String;
  access: Longword;

begin
  aName := '{#emit MyDriverName}';
  dName := '{#emit MyDriverDisplayName}';
  fName := '{#emit MyDriverFileName}';
  sPath := ExpandConstant('{sys}') + '\drivers\';

  access := (SERVICE_START + DELETE_ACCESS_RIGHT + SERVICE_STOP + SERVICE_INTERROGATE);

  if DeviceExists(aName, access) = False then
    begin
      if SimpleCreateDevice(aName, dName, sPath + fName, SERVICE_DEMAND_START,
        access, SERVICE_ERROR_IGNORE, False) = True then
        begin
          SimpleStartDevice(aName, access, True, True);
        end
    end
  else
    begin
      UninstallDevice();
      if SimpleCreateDevice(aName, dName, sPath + fName, SERVICE_DEMAND_START,
        access, SERVICE_ERROR_IGNORE, False) = True then
        begin
          SimpleStartDevice(aName, access, True, True);
        end;
    end;
end;

procedure CurStepChanged(CurStep: TSetupStep);
begin
  if CurStep = ssPostInstall then
  begin
    InstallDevice();
  end;
end;

procedure CurUninstallStepChanged(CurUninstallStep: TUninstallStep);
begin
  case CurUninstallStep of
    usUninstall:
      begin
         UninstallDevice();
      end;
  end;
end;
